@@ -7,13 +7,16 @@ import warnings
 import numpy as np
 from tqdm import tqdm
 from scipy.constants import speed_of_light
-from scipy.ndimage.filters import convolve
-from scipy.interpolate import interp1d
+from scipy.ndimage.filters import convolve, gaussian_filter1d
+from scipy.interpolate import interp1d, UnivariateSpline
 
 
 from . import broadening
 from .atmosphere.interpolation import AtmosphereInterpolator
-from .continuum_and_radial_velocity import match_rv_continuum
+from .continuum_and_radial_velocity import (
+    match_rv_continuum,
+    apply_radial_velocity_and_continuum,
+)
 from .large_file_storage import setup_lfs
 from .iliffe_vector import Iliffe_vector
 from .nlte import update_nlte_coefficients
@@ -163,20 +166,15 @@ class Synthesizer:
         return segments
 
     def apply_radial_velocity_and_continuum(
-        self, wave, wmod, smod, cmod, vrad, cscale, segments
+        self, wave, spec, wmod, smod, cmod, vrad, cscale, cscale_type, segments
     ):
-        for il in segments:
-            if vrad[il] is not None:
-                rv_factor = np.sqrt((1 + vrad[il] / clight) / (1 - vrad[il] / clight))
-                wmod[il] *= rv_factor
-            smod[il] = np.interp(wave[il], wmod[il], smod[il])
-            if cmod is not None:
-                cmod[il] = np.interp(wave[il], wmod[il], cmod[il])
-
-            if cscale[il] is not None and not np.all(cscale[il] == 0):
-                x = wave[il] - wave[il][0]
-                smod[il] *= np.polyval(cscale[il], x)
-        return smod
+        smod = apply_radial_velocity_and_continuum(
+            wave, wmod, smod, vrad, cscale, cscale_type, segments
+        )
+        cmod = apply_radial_velocity_and_continuum(
+            wave, wmod, cmod, vrad, None, None, segments
+        )
+        return smod, cmod
 
     def integrate_flux(self, mu, inten, deltav, vsini, vrt, osamp=1):
         """
@@ -561,8 +559,8 @@ class Synthesizer:
         else:
             raise ValueError("Radial Velocity mode not understood")
 
-        smod = self.apply_radial_velocity_and_continuum(
-            wave, wmod, smod, cmod, vrad, cscale, segments
+        smod, cmod = self.apply_radial_velocity_and_continuum(
+            wave, sme.spec, wmod, smod, cmod, vrad, cscale, sme.cscale_type, segments
         )
 
         # Merge all segments
